/**
  *  \file planetscentral/dbclient/dbsubtree.h
  *  \brief Database Client: Subtrees
  */
#ifndef C2SERVER_PLANETSCENTRAL_DBCLIENT_DBSUBTREE_H
#define C2SERVER_PLANETSCENTRAL_DBCLIENT_DBSUBTREE_H

#include "planetscentral/dbclient/dbclient.h"
#include "planetscentral/dbclient/dbhash.h"
#include "planetscentral/dbclient/dbint.h"
#include "planetscentral/dbclient/dbset.h"
#include "planetscentral/dbclient/dbstring.h"
#include "planetscentral/dbclient/dblist.h"
#include "planetscentral/dbclient/dbzset.h"

/** A subtree of the redis keyspace. By convention, redis keys are split into components
    at ":", forming a tree hierarchy much like file names in a file system. This class
    represents a virtual root in that tree, allowing access to subtrees and keys. */
class DbSubtree {
 public:
    DbSubtree(DbClient& client, string_t prefix);
    DbSubtree(DbClient& client, const char* prefix);
    ~DbSubtree();

    DbHash       hashKey(const string_t& name) const;
    DbHash       hashKey(const char* name) const;
    DbInt        intKey(const string_t& name) const;
    DbInt        intKey(const char* name) const;
    DbIntSet     intSetKey(const string_t& name) const;
    DbIntSet     intSetKey(const char* name) const;
    DbStringSet  stringSetKey(const string_t& name) const;
    DbStringSet  stringSetKey(const char* name) const;
    DbZSet       zsetKey(const string_t& name) const;
    DbZSet       zsetKey(const char* name) const;
    DbString     stringKey(const string_t& name) const;
    DbString     stringKey(const char* name) const;
    DbIntList    intListKey(const string_t& name) const;
    DbIntList    intListKey(const char* name) const;
    DbStringList stringListKey(const string_t& name) const;
    DbStringList stringListKey(const char* name) const;

    DbSubtree    subtree(const string_t& name) const;
    DbSubtree    subtree(const char* name) const;
    DbSubtree    subtree(int32_t name) const;

    string_t getPrefix() const
        { return prefix; }
    DbClient& getClient()
        { return client; }

 private:
    DbClient& client;
    string_t prefix;
};

#endif
