/**
  *  \file planetscentral/dirinfo.h
  *  \brief File Server - Directory Game Information
  */
#ifndef C2SERVER_PLANETSCENTRAL_DIRINFO_H
#define C2SERVER_PLANETSCENTRAL_DIRINFO_H

#include <list>
#include "game/reg.h"
#include "game/load.h"
#include "game/racename.h"
#include "game/storage/overview.h"

class Directory;

/** Directory Game Information. This class contains game-related
    information (as opposed to file-system related information in
    UserDirectory) for a directory. */
class DirInfo {
 public:
    typedef std::list<string_t> MissingList_t;

    DirInfo(Directory& dir);
    ~DirInfo();

    bool haveRegInfo() const;
    const GRegInfo& getRegInfo() const;
    const string_t& getRegFileName() const;

    bool haveGame() const;
    const GDirectoryOverview& getOverview() const;

    bool haveRaceNames() const;
    const GRaceNameList& getRaceNames() const;

    const MissingList_t& getMissingFiles() const;

 private:
    void init(Directory& dir);

    GRegInfo reg_info;
    string_t reg_file_name;

    GDirectoryOverview overview;

    bool have_race_names;
    GRaceNameList race_names;

    MissingList_t missing_files;
};

#endif
