/**
  *  \file planetscentral/host/host.h
  */
#ifndef C2SERVER_PLANETSCENTRAL_HOST_HOST_H
#define C2SERVER_PLANETSCENTRAL_HOST_HOST_H

#include "planetscentral/dbclient/dbclient.h"
#include "planetscentral/dbclient/connection.h"
#include "planetscentral/host/game.h"

class Runner;
class Mutex;
class Cron;
class IntArgBlock;
class ReconnectingConnection;
class RouterClient;
class MailClient;

class HostConnection : public Connection {
 public:
    HostConnection();

    void handleRequest();
    void handleCommand(const string_t& cmd, IntArgBlock& args);

    bool isAdmin() const;
    void checkAdmin();
    void checkPermission(Game& g, Game::PermissionLevel level);
    string_t getUser() const;

 private:
    string_t user;
};

// Runners. Internally synchronized.
extern Runner host_runner;
extern Runner checkturn_runner;

extern Mutex output_mutex;

// Database connection. Should have database_rc when you use it.
extern DbReadyClient database_connection;
extern ReconnectingConnection database_rc;

// Filer connection. Should have file_rc when you use it.
// If you also use the database, you must take that first.
extern DbClient file_connection;
extern ReconnectingConnection file_rc;

// User filer connection. Should have userfile_rc when you use it.
// If you also use the database, you must take that first.
extern DbClient userfile_connection;
extern ReconnectingConnection userfile_rc;

// Mailer connection. Should have mail_rc when you use it.
// If you also use the database, you must take that first.
extern MailClient mail_connection;
extern ReconnectingConnection mail_rc;

// Forum connection. Should have talk_rc when you use it.
// If you also use the database, you must take that first.
extern DbClient talk_connection;
extern ReconnectingConnection talk_rc;

// Cron.
extern Cron cron;

// Router
extern RouterClient router_client;

#endif
