/**
  *  \file planetscentral/host/schedule.h
  *  \brief Host: Scheduler operations
  */
#ifndef C2SERVER_PLANETSCENTRAL_HOST_SCHEDULE_H
#define C2SERVER_PLANETSCENTRAL_HOST_SCHEDULE_H

#include <time.h>
#include "cpluslib/smallset.h"
#include "cpluslib/types.h"

class DbHash;
class DbSubtree;

class Schedule {
 public:
    enum Type {
        Stopped,
        Weekly,
        Daily,
        Quick,
        Manual
    };
    enum Condition {
        NoCondition,
        TurnCondition,
        TimeCondition
    };

    Schedule();

    void setType(Type t);
    void setWeekDays(SmallSet<int8_t> weekdays);
    void setInterval(int interval);
    void setDaytime(int daytime);
    void setHostEarly(bool value);
    void setHostDelay(int32_t minutes);
    void setHostLimit(int32_t minutes);
    void setCondition(Condition cond, int32_t arg);

    Type getType() const;
    SmallSet<int8_t> getWeekDays() const;
    int getInterval() const;
    int getDaytime() const;
    bool getHostEarly() const;
    int32_t getHostDelay() const;
    int32_t getHostLimit() const;
    Condition getCondition() const;
    int32_t getConditionArg() const;

    bool isExpired(int32_t turn, int32_t time) const;

    void loadFrom(DbHash h);
    void saveTo(DbHash h);

    int32_t getNextHost(int32_t now) const;
    int32_t getPreviousHost(int32_t now) const;
    int32_t getPreviousVirtualHost(int32_t now) const;

 private:
    bool dirty;

    // Variable name is the same as DB field name
    Type type;
    SmallSet<int8_t> weekdays;
    int interval;
    int32_t daytime;
    bool hostEarly;
    int32_t hostDelay;
    int32_t hostLimit;
    Condition condition;
    int32_t condTurnOrTime;     // condTurn or condTime
};

int32_t pickDayTime(DbSubtree& root);

int32_t getCurrentTime();
int     getWeekdayFromTime(int32_t time);
time_t  getSystemTimeFromTime(int32_t t);
int32_t getUserTimeFromTime(int32_t time);

void    setTimeScale(int scale);

#endif
